@@ -1,6 +1,6 @@
 ;;; tramp-gvfs.el --- Tramp access functions for GVFS daemon
 
-;; Copyright (C) 2009 Free Software Foundation, Inc.
+;; Copyright (C) 2009, 2010 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -196,7 +196,7 @@
 
 ;; <interface name='org.gtk.vfs.MountOperation'>
 ;;   <method name='askPassword'>
-;;     <arg name='message' type='s'  direction='in'/>
+;;     <arg name='message'        type='s' direction='in'/>
 ;;     <arg name='default_user'   type='s' direction='in'/>
 ;;     <arg name='default_domain' type='s' direction='in'/>
 ;;     <arg name='flags'          type='u' direction='in'/>
@@ -499,9 +499,10 @@ is no information where to trace the message.")
 
 (defun tramp-gvfs-dbus-event-error (event err)
   "Called when a D-Bus error message arrives, see `dbus-event-error-hooks'."
-;  (tramp-cleanup-connection tramp-gvfs-dbus-event-vector)
-  (tramp-message tramp-gvfs-dbus-event-vector 1 "%S" event)
-  (tramp-error tramp-gvfs-dbus-event-vector 'file-error "%s" (cadr err)))
+  (when tramp-gvfs-dbus-event-vector
+    ;(tramp-cleanup-connection tramp-gvfs-dbus-event-vector)
+    (tramp-message tramp-gvfs-dbus-event-vector 10 "%S" event)
+    (tramp-error tramp-gvfs-dbus-event-vector 'file-error "%s" (cadr err))))
 
 (add-hook 'dbus-event-error-hooks 'tramp-gvfs-dbus-event-error)
 
@@ -726,6 +727,10 @@ is no information where to trace the message.")
 	       (signal (car err) (cdr err)))
 	   (delete-file tmpfile)))))
 
+    ;; Set file modification time.
+    (when (or (eq visit t) (stringp visit))
+      (set-visited-file-modtime (nth 5 (file-attributes filename))))
+
     ;; The end.
     (when (or (eq visit t) (null visit) (stringp visit))
       (tramp-message v 0 "Wrote %s" filename))
@@ -1052,8 +1057,7 @@ connection if a previous connection has died for some reason."
 	 vec 3 "Opening connection for %s@%s using %s..." user host method))
 
       ;; Enable auth-sorce and password-cache.
-      (tramp-set-connection-property
-       (tramp-get-connection-process vec) "first-password-request" t)
+      (tramp-set-connection-property vec "first-password-request" t)
 
       ;; There will be a callback of "askPassword", when a password is
       ;; needed.
@@ -1092,7 +1096,7 @@ connection if a previous connection has died for some reason."
 	      vec 'file-error
 	      "Timeout reached mounting %s@%s using %s" user host method)))
 	(while (not (tramp-get-file-property vec "/" "fuse-mountpoint" nil))
-	  (sit-for 0.1)))
+	  (read-event nil nil 0.1)))
 
       ;; We set the connection property "started" in order to put the
       ;; remote location into the cache, which is helpful for further
